package com.css.simulation.resource.scheduler.service;

import api.common.pojo.constants.DictConstants;
import api.common.util.*;
import com.css.simulation.resource.scheduler.manager.TaskIndexManager;
import com.css.simulation.resource.scheduler.manager.TaskManager;
import com.css.simulation.resource.scheduler.mapper.IndexTemplateMapper;
import com.css.simulation.resource.scheduler.mapper.ProjectMapper;
import com.css.simulation.resource.scheduler.mapper.TaskIndexMapper;
import com.css.simulation.resource.scheduler.mapper.TaskMapper;
import com.css.simulation.resource.scheduler.pojo.po.IndexTemplatePO;
import com.css.simulation.resource.scheduler.pojo.po.ProjectPO;
import com.css.simulation.resource.scheduler.pojo.po.TaskIndexPO;
import com.css.simulation.resource.scheduler.pojo.po.TaskPO;
import com.css.simulation.resource.scheduler.pojo.to.ScoreTO;
import com.css.simulation.resource.scheduler.util.MinioUtil;
import io.minio.MinioClient;
import lombok.SneakyThrows;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.data.redis.core.StringRedisTemplate;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import java.util.*;
import java.util.stream.Collectors;

@Service
@Slf4j
public class TaskService {

    private final String USER_ID = "simulation-resource-scheduler";
    @Autowired
    MinioClient minioClient;
    @Value("${minio.bucket-name}")
    String bucketName;
    @Autowired
    StringRedisTemplate redisTemplate;
    @Autowired
    ProjectMapper projectMapper;
    @Autowired
    TaskManager taskManager;
    @Autowired
    TaskMapper taskMapper;
    @Autowired
    TaskIndexManager taskIndexManager;
    @Autowired
    TaskIndexMapper taskIndexMapper;
    @Autowired
    IndexTemplateMapper indexTemplateMapper;
    @Value("${scheduler.manual-project.topic}")
    String manualProjectTopic;
    @Value("${scheduler.score.hostname}")
    String hostname;
    @Value("${scheduler.score.username}")
    String username;
    @Value("${scheduler.score.password}")
    String password;
    @Value("${scheduler.score.py-path}")
    String pyPath;
    @Value("${scheduler.linux-temp-path}")
    String linuxTempPath;

    public void taskTick(String taskId) {
        log.info("------- /tick 接收到任务 " + taskId + "的心跳！");
        // 刷新 redis 心跳时间
        ProjectPO projectPO = projectMapper.selectById(taskId);
        String projectId = projectPO.getId();
        redisTemplate.opsForValue().set(manualProjectTopic + ":" + projectId + ":" + taskId, TimeUtil.getNowString());
    }

    @Transactional
    @SneakyThrows
    public void taskState(String taskId, String state) {
        log.info("------- /state 接收到任务 " + taskId + "的状态：" + state);
        //1 根据 taskId 修改任务状态 taskState。
        taskMapper.updateState(taskId, state);
        //2 如果 taskState 为完成状态，校验一下项目的已完成数量。
        ProjectPO projectPO = projectMapper.selectById(taskId);
        String projectId = projectPO.getId();
        String scenePackageId = projectPO.getScenePackageId();  // 场景测试包 id，指标根 id
        int taskNum = projectMapper.selectTaskNumById(projectId);
        int completedTaskNum = projectMapper.selectTaskNumByProjectIdAndState(projectId, DictConstants.TASK_COMPLETED);
        if (taskNum != completedTaskNum) {  // 已完成任务数等于所有任务数量，才会准备打分；否则退出。
            return;
        }
        List<TaskPO> taskList = taskMapper.selectTaskListByProjectId(projectId);  // 所有任务信息
        // -------------------------------- 查询叶子指标 --------------------------------
        List<IndexTemplatePO> leafIndexTemplateList = indexTemplateMapper.selectLeafIndexWithRuleDetailsByPackageId(scenePackageId);
        List<TaskIndexPO> leafTaskIndexList = new ArrayList<>();
        for (int i = 0; i < leafIndexTemplateList.size(); i++) {
            IndexTemplatePO indexTemplatePO = leafIndexTemplateList.get(i);
            // -------------------------------- 将叶子节点对应的打分规则保存到临时目录 --------------------------------
            String ruleName = indexTemplatePO.getRuleName();    // 打分脚本名称，例如 AEB_1-1
            String ruleDetails = indexTemplatePO.getRuleDetails();    // 打分脚本内容
            String ruleDetailsPath = pyPath + "script/" + projectId + "_" + i;
            FileUtil.writeInputStreamToLocalFile(IoUtil.stringToInputStream(ruleDetails), ruleDetailsPath);
            // -------------------------------- 查询每个叶子指标包括的场景 --------------------------------
            Set<String> sceneIdSet = new HashSet<>();
            String naturalIds = indexTemplatePO.getSceneNaturalIds();
            String standardIds = indexTemplatePO.getSceneStatueIds();
            String accidentIds = indexTemplatePO.getSceneTrafficIds();
            if (StringUtil.isNotEmpty(naturalIds)) {
                String[] naturalIdArray = naturalIds.split(",");
                sceneIdSet.addAll(Arrays.asList(naturalIdArray));
            }
            if (StringUtil.isNotEmpty(standardIds)) {
                String[] standardArray = standardIds.split(",");
                sceneIdSet.addAll(Arrays.asList(standardArray));
            }
            if (StringUtil.isNotEmpty(accidentIds)) {
                String[] accidentIdArray = accidentIds.split(",");
                sceneIdSet.addAll(Arrays.asList(accidentIdArray));
            }
            int resultNumberOfCurrentIndex = sceneIdSet.size();
            // -------------------------------- 计算叶子指标的得分 --------------------------------
            double sum = taskList.stream()
                    .filter(task1 -> sceneIdSet.contains(task1.getSceneId()))
                    .mapToDouble(task2 -> {
                        // 计算每个任务的得分
                        ScoreTO score;
                        String runResultMinio = task2.getRunResult();
                        String runResultLinux = linuxTempPath + runResultMinio;
//                        String command = "python3 " + pyPath + " " + runResultLinux + " " + task2.getSceneType();  // 默认使用场景名称找打分脚本
                        String command = "python3 " + pyPath + "main.py " + runResultLinux + " " + task2.getSceneType() + " " + ruleDetailsPath; // 指定打分脚本

                        try {
                            MinioUtil.downloadToFile(minioClient, bucketName, runResultMinio, runResultLinux);
                            log.info("------- 开始执行打分命令：" + command);
                            score = JsonUtil.jsonToBean(SshUtil.execute(hostname, username, password, command), ScoreTO.class);
                            log.info("------- 打分结束，结果为：" + score);
                        } catch (Exception e) {
                            log.error("------- 打分出错，命令为：" + command);
                            throw new RuntimeException(e.getMessage());
                        }
                        task2.setReturnSceneId(score.getUnit_scene_ID());
                        task2.setScore(score.getUnit_scene_score());
                        task2.setTargetEvaluate(score.getEvaluate_item());
                        task2.setScoreExplain(score.getScore_description());
                        task2.setModifyUserId(USER_ID);
                        task2.setModifyTime(TimeUtil.getNowForMysql());
                        return score.getUnit_scene_score();
                    }).sum();
            // 计算不合格的任务数（不到100分就是不合格）
            long notStandardSceneNum = taskList.stream().filter(task1 -> sceneIdSet.contains(task1.getSceneId()) && task1.getScore() < 100).count();
            // 叶子指标的得分（叶子指标下所有场景得分的平均值）
            double leafIndexScore = sum / resultNumberOfCurrentIndex;
            // -------------------------------- 保存叶子指标得分 --------------------------------
            indexTemplatePO.setTempScore(leafIndexScore);
            TaskIndexPO leafTaskIndex = TaskIndexPO.builder()
                    .id(StringUtil.getRandomUUID())
                    .pId(projectId)
                    .target(indexTemplatePO.getIndexId())
                    .notStandardSceneNum((int) notStandardSceneNum)
                    .score(leafIndexScore)
                    .build();
            leafTaskIndex.setCreateUserId(USER_ID);
            leafTaskIndex.setCreateTime(TimeUtil.getNowForMysql());
            leafTaskIndex.setModifyUserId(USER_ID);
            leafTaskIndex.setModifyTime(TimeUtil.getNowForMysql());
            leafTaskIndex.setIsDeleted("0");
            leafTaskIndexList.add(leafTaskIndex);
        }

        // 根据每个指标的得分和权重算出 project 的总得分。
        double totalScore = compute(leafIndexTemplateList);
        // 保存分数
        // 保存任务分数
        taskManager.batchUpdateByScoreResult(taskList);
        // 保存指标分数
        taskIndexManager.batchInsertLeafIndex(leafTaskIndexList);
        // 保存总分数
        TaskIndexPO totalTaskIndex = TaskIndexPO.builder()
                .id(StringUtil.getRandomUUID())
                .pId(projectId)
                .target(scenePackageId)
                .score(totalScore)
                .build();
        totalTaskIndex.setCreateUserId(USER_ID);
        totalTaskIndex.setCreateTime(TimeUtil.getNowForMysql());
        totalTaskIndex.setModifyUserId(USER_ID);
        totalTaskIndex.setModifyTime(TimeUtil.getNowForMysql());
        totalTaskIndex.setIsDeleted("0");
        taskIndexMapper.insertTotalIndex(totalTaskIndex);
    }


    public Boolean taskConfirm(String taskId) {
        // 查询 task 如果不是 pending 则不执行
        String state = taskMapper.selectStateById(taskId);
        return DictConstants.TASK_PENDING.equals(state);
    }

    public double compute(List<IndexTemplatePO> leaf) {
        double result = 0.0;
        Map<String, List<IndexTemplatePO>> groups = leaf.stream().collect(Collectors.groupingBy(IndexTemplatePO::getParentId));
        Set<String> idSet = groups.keySet();
        List<IndexTemplatePO> indexTemplatePOList = indexTemplateMapper.selectByIdList(CollectionUtil.setToList(idSet));
        indexTemplatePOList.forEach(index1 -> {
            double sum = groups.get(index1.getIndexId()).stream().mapToDouble(index2 -> index2.getTempScore() * Double.parseDouble(index2.getWeight())).sum();
            index1.setTempScore(sum);
        });
        if (indexTemplatePOList.size() > 1) {
            result = result + compute(indexTemplatePOList);
        }
        return result;
    }
}
